var net = require('net');
var url = require("url");
var auth = require("basic-auth");

function handleAuthProxy(_req, _res, _next, _apiProxy, _logger) {
	if (_req.url.length < 3) {  //we need at least <domain>/<command>
		_next();
		return;
	}
	
	var indexCommand = _req.url.lastIndexOf("/");
	if (indexCommand == -1) {
		_next();
		return;
	}

	var eolMarker = "\r\n\r\n"; 
	var domain = _req.url.substring(1, indexCommand);
	var command = _req.url.substring(indexCommand + 1);

	if (domain != _apiProxy.pseudoDomain) {
		_next();
		return;
	}

	var credentials = auth(_req);
	if (!credentials || credentials.name !== _apiProxy.userName || 
			credentials.pass !== _apiProxy.password) {
	    _res.statusCode = 401;
	    _res.setHeader('WWW-Authenticate', 'Basic realm="EvoWebServer"');
	    _res.end('Access denied');
	    return;
  	}
	
	var conn = net.createConnection(_apiProxy.port, _apiProxy.address);
	conn.once('connect', () => {
		_logger.log('debug', "AuthProxy: connected");
		//send message to EMS
		conn.write("GET /" + command + " HTTP/1.1" + eolMarker);
	});
	conn.on('timeout', () => {
		_logger.log('debug', "AuthProxy: timeout");
		_next();
		return;
	});
	conn.on('error', (error) => {
		_logger.log('debug', "AuthProxy: error", error);
		_next();
		return;
	});
	var emsMsg = [];
	conn.on('data', (data) => {
		//data coming from EMS
		emsMsg.push(data);
	});
	conn.on('end', () => {
		var ds = emsMsg.toString();
		//remove the response header
		var index = ds.indexOf(eolMarker);
		var messageBody;
		if (index > -1) {
			var headerSize = index + eolMarker.length;
			messageBody = ds.substring(headerSize);
		} else {
			messageBody = ds;
		}
		_res.writeHead(200, 'text/plain');
		_res.end(messageBody);
		msg=[];
	});
}

exports.handle = function(_apiProxy, _logger) {
	return function(_req, _res, _next) {
		handleAuthProxy(_req, _res, _next, _apiProxy, _logger);
	}
};